#include <Windows.h>
#include <Psapi.h>
#include <string>
#include <boost/filesystem.hpp>
#include "injector.hpp"

namespace
{
const uint32_t max_modules = 1024;
const std::wstring skype_preview_dll(L"SkypePreview.dll");
}

const std::wstring get_skype_path()
{
	HKEY key;
	std::wstring ret;
	if(ERROR_SUCCESS != RegOpenKeyExW(HKEY_LOCAL_MACHINE, L"SOFTWARE\\Skype\\Phone", 0, KEY_QUERY_VALUE, &key))
		return ret;

	DWORD type = REG_SZ;
	DWORD len = 0;
	if(ERROR_SUCCESS != RegQueryValueExW(key, L"SkypePath", 0, &type, 0, &len)
		|| type != REG_SZ)
	{
		RegCloseKey(key);
		return ret;
	}

	ret.resize(len);
	if(ERROR_SUCCESS != RegQueryValueExW(key, L"SkypePath", 0, &type, reinterpret_cast<LPBYTE>(&ret[0]), &len))
	{
		RegCloseKey(key);
		return ret;
	}

	if(ret.back() == L'\0')
		ret.pop_back();

	return ret;
}

const std::wstring get_full_skype_preview_dll_path()
{
	wchar_t my_module_name[MAX_PATH];
	if(GetModuleFileNameW(GetModuleHandleW(0), my_module_name, sizeof(my_module_name) / sizeof(wchar_t)))
	{
		boost::filesystem::wpath my_path(my_module_name);
		my_path.remove_filename();
		my_path /= skype_preview_dll;
		return my_path.string<std::wstring>();
	}

	return skype_preview_dll;
}

int CALLBACK WinMain(HINSTANCE hInstance, HINSTANCE /*hPrevInstance*/, LPSTR /*lpCmdLine*/, int /*nCmdShow*/)
{
	const std::wstring skype_path(get_skype_path());
	if(skype_path.empty())
	{
		MessageBoxW(0, L"Skype is not installed on your computer.", L"Error", MB_ICONEXCLAMATION | MB_OK);
		return -1;
	}

	std::wstring full_skype_preview_dll_path(get_full_skype_preview_dll_path());

	boost::filesystem::wpath skype_dir(skype_path);
	skype_dir.remove_filename();

	//If Skype is already open...
	try
	{
		uint32_t skype_pid = injector::find_process_by_name(L"Skype.exe");
		if(skype_pid)
		{
			HANDLE skype_process = OpenProcess(PROCESS_QUERY_INFORMATION |
				PROCESS_VM_READ,
				FALSE, skype_pid);
			if(skype_process != 0)
			{
				HMODULE modules[max_modules];
				DWORD needed;
				bool found_skype_preview = false;
				if(EnumProcessModules(skype_process, modules, sizeof(modules), &needed))
				{
					needed /= sizeof(HMODULE);
					for(uint32_t i = 0; i != needed; ++i)
					{
						wchar_t module_name[MAX_PATH];

						if(GetModuleFileNameEx(skype_process, modules[i], module_name,
							sizeof(module_name) / sizeof(wchar_t)))
						{
							boost::filesystem::path module_path(module_name);
							if(module_path.filename() == skype_preview_dll)
							{
								found_skype_preview = true;
								break;
							}
						}
					}
				}

				CloseHandle(skype_process);

				if(!found_skype_preview)
				{
					try
					{
						injector inj;
						inj.set_blocking(false);
						inj.inject(skype_pid, full_skype_preview_dll_path);
					}
					catch(const injector_exception&)
					{
						MessageBoxW(0, L"Unable to load Skype Preview plugin", L"Error", MB_ICONEXCLAMATION | MB_OK);
						return -1;
					}
				}

				return 0;
			}
		}
	}
	catch(const injector_exception&)
	{
	}

	STARTUPINFO startup_info = {};
	startup_info.cb = sizeof(startup_info);
	PROCESS_INFORMATION process_info = {};
	if(!CreateProcessW(skype_path.c_str(), 0, 0, 0, FALSE, CREATE_SUSPENDED,
		0, skype_dir.string<std::wstring>().c_str(), &startup_info, &process_info))
	{
		MessageBoxW(0, L"Unable to start Skype", L"Error", MB_ICONEXCLAMATION | MB_OK);
		return -1;
	}

	try
	{
		injector inj;
		inj.set_blocking(false);
		inj.inject(process_info.dwProcessId, full_skype_preview_dll_path);
	}
	catch(const injector_exception&)
	{
		TerminateProcess(process_info.hProcess, 0);
		CloseHandle(process_info.hProcess);
		CloseHandle(process_info.hThread);
		MessageBoxW(0, L"Unable to load Skype Preview plugin", L"Error", MB_ICONEXCLAMATION | MB_OK);
		return -1;
	}

	ResumeThread(process_info.hThread);

	CloseHandle(process_info.hProcess);
	CloseHandle(process_info.hThread);

	return 0;
}
