#include <boost/scoped_array.hpp>
#include "menu.h"
#include "sp_exception.h"

menu::submenu::submenu(uint16_t id, const std::wstring& text)
	:id_(id),
	text_(text)
{
}

uint32_t menu::submenu::get_id() const
{
	return id_;
}

const std::wstring& menu::submenu::get_text() const
{
	return text_;
}

void menu::add_menu(HWND window, const std::wstring& text, const submenu_list& sumbenus)
{
	HMENU handle = GetMenu(window);
	if(!handle)
		throw sp_exception(sp_exception::internal_error, L"Window has no menubar");

	HMENU popup = CreatePopupMenu();
	if(!popup)
		throw sp_exception(sp_exception::internal_error, L"Unable to create popup menu");

	if(!AppendMenuW(handle, MF_STRING | MF_POPUP, reinterpret_cast<UINT_PTR>(popup), text.c_str()))
	{
		DestroyMenu(popup);
		throw sp_exception(sp_exception::internal_error, L"Unable to add popup menu");
	}

	for(submenu_list::const_iterator it = sumbenus.begin(); it != sumbenus.end(); ++it)
	{
		if(!AppendMenuW(popup, MF_STRING, (*it).get_id(), (*it).get_text().c_str()))
		{
			DestroyMenu(popup);
			throw sp_exception(sp_exception::internal_error, L"Unable to add popup menu item");
		}
	}
}

bool menu::has_menu(HWND window, const std::wstring& text)
{
	HMENU handle = GetMenu(window);
	if(!handle)
		return false;
	
	uint32_t i = 0;

	while(true)
	{
		MENUITEMINFOW info = {};
		info.cbSize = sizeof(info);
		info.fMask = MIIM_STRING;
		if(!GetMenuItemInfoW(handle, i, TRUE, &info))
			break;

		boost::scoped_array<wchar_t> buf(new wchar_t[++info.cch]);
		info.dwTypeData = buf.get();
		if(!GetMenuItemInfoW(handle, i, TRUE, &info))
			break;

		if(buf.get() == text)
			return true;

		++i;
	}

	return false;
}
