#pragma once
#include <string>
#include <boost/noncopyable.hpp>
#include <boost/thread/mutex.hpp>
#include <boost/thread/thread.hpp>
#include <boost/thread/condition_variable.hpp>
#include <Windows.h>
#include <GdiPlus.h>
#include <memory>
#include "configuration.h"
#include "unpacked_image.h"

class web_content_loader;
class image_window;

class url_processor : boost::noncopyable
{
public:
	static void set_wait_image(Gdiplus::Image* image);

	static void set_max_sizes(uint32_t max_size, uint32_t max_gif_size);

	static void set_max_title_length(uint32_t max_title_length);

	static void set_default_encoding(const std::string& default_encoding);

	static void set_proxy(const std::string& proxy);
	static void set_auto_proxy(bool auto_proxy);
	static void set_no_proxy(bool no_proxy);
	static void set_proxy_auth(bool use_auth, const std::string& login, const std::string& password);

	static void set_contains_filter(bool filter_on, const configuration::string_set& strings);
	static void set_does_not_contain_filter(bool filter_on, const configuration::string_set& strings);

	static void enable_preview(bool preview_images, bool preview_titles);

	static void set_work_item(const std::wstring& url, int32_t x, int32_t y, bool active_cursor);

	static void start(web_content_loader& loader, image_window& displayer);

	static void wait();

private:
	url_processor();

	static void process_url(web_content_loader& loader, image_window& displayer,
		const std::wstring& url, int32_t x, int32_t y);
	
	static void monitor_thread(web_content_loader& loader, image_window& displayer);

	static boost::condition_variable worker_var_;
	static boost::mutex worker_mutex_;
	static std::wstring last_url_, prev_url_;
	static uint32_t max_size_, max_gif_size_, max_title_length_;

	static web_content_loader* loader_;
	static image_window* displayer_;

	static boost::thread processor_thread_;

	static int32_t last_x_, last_y_;

	static bool active_cursor_;

	static std::auto_ptr<unpacked_image> wait_image_;

	static std::string default_encoding_;
	static std::string proxy_;
	static bool auto_proxy_, no_proxy_, use_proxy_auth_;
	static std::string proxy_login_, proxy_password_;

	static bool preview_images_, preview_titles_;

	static bool contains_filter_on_, does_not_contain_filter_on_;
	static configuration::string_set contains_filter_strings_, does_not_contain_filter_strings_;
};
