#include <Windows.h>
#include <winhttp.h>
#include <Psapi.h>
#include <cmath>
#include <sstream>
#include <CommCtrl.h>
#include <boost/scoped_array.hpp>
#include <boost/algorithm/string.hpp>
#include <boost/algorithm/string/regex.hpp>
#include <boost/lexical_cast.hpp>
#include "utility.h"
#include "byte_stream.h"
#include "configuration.h"
#include "istream_owner_image.h"
#include "sp_exception.h"
#include "../libiconv/include/iconv.h"

HCURSOR utility::hand_cursor_ = LoadCursorW(NULL, IDC_HAND);
boost::regex utility::title_regex_("<title[^<>]*?>(.*?)</title>", boost::regex::icase | boost::regex::perl);
boost::regex utility::header_charset_regex_("\\sContent-Type: .+?; charset=(.+?)\\s", boost::regex::icase);
boost::regex utility::meta_charset_regex_("<meta\\s+http-equiv=\"Content-Type\"\\s*content=\".+?;\\s*charset=(.+?)\"", boost::regex::icase);
boost::wregex utility::space_regex_(L"[\r\n\t]");
const utility::html_entity_map utility::html_entities_(utility::init_html_entities());

const utility::html_entity_map utility::init_html_entities()
{
	html_entity_map ret;
	ret.insert(std::make_pair(L"quot", 0x0022));
	ret.insert(std::make_pair(L"amp", 0x0026));
	ret.insert(std::make_pair(L"apos", 0x0027));
	ret.insert(std::make_pair(L"lt", 0x003C));
	ret.insert(std::make_pair(L"gt", 0x003E));
	ret.insert(std::make_pair(L"nbsp", 0x00A0));
	ret.insert(std::make_pair(L"iexcl", 0x00A1));
	ret.insert(std::make_pair(L"cent", 0x00A2));
	ret.insert(std::make_pair(L"pound", 0x00A3));
	ret.insert(std::make_pair(L"curren", 0x00A4));
	ret.insert(std::make_pair(L"yen", 0x00A5));
	ret.insert(std::make_pair(L"brvbar", 0x00A6));
	ret.insert(std::make_pair(L"sect", 0x00A7));
	ret.insert(std::make_pair(L"uml", 0x00A8));
	ret.insert(std::make_pair(L"copy", 0x00A9));
	ret.insert(std::make_pair(L"ordf", 0x00AA));
	ret.insert(std::make_pair(L"laquo", 0x00AB));
	ret.insert(std::make_pair(L"not", 0x00AC));
	ret.insert(std::make_pair(L"shy", 0x00AD));
	ret.insert(std::make_pair(L"reg", 0x00AE));
	ret.insert(std::make_pair(L"macr", 0x00AF));
	ret.insert(std::make_pair(L"deg", 0x00B0));
	ret.insert(std::make_pair(L"plusmn", 0x00B1));
	ret.insert(std::make_pair(L"sup2", 0x00B2));
	ret.insert(std::make_pair(L"sup3", 0x00B3));
	ret.insert(std::make_pair(L"acute", 0x00B4));
	ret.insert(std::make_pair(L"micro", 0x00B5));
	ret.insert(std::make_pair(L"para", 0x00B6));
	ret.insert(std::make_pair(L"middot", 0x00B7));
	ret.insert(std::make_pair(L"cedil", 0x00B8));
	ret.insert(std::make_pair(L"sup1", 0x00B9));
	ret.insert(std::make_pair(L"ordm", 0x00BA));
	ret.insert(std::make_pair(L"raquo", 0x00BB));
	ret.insert(std::make_pair(L"frac14", 0x00BC));
	ret.insert(std::make_pair(L"frac12", 0x00BD));
	ret.insert(std::make_pair(L"frac34", 0x00BE));
	ret.insert(std::make_pair(L"iquest", 0x00BF));
	ret.insert(std::make_pair(L"Agrave", 0x00C0));
	ret.insert(std::make_pair(L"Aacute", 0x00C1));
	ret.insert(std::make_pair(L"Acirc", 0x00C2));
	ret.insert(std::make_pair(L"Atilde", 0x00C3));
	ret.insert(std::make_pair(L"Auml", 0x00C4));
	ret.insert(std::make_pair(L"Aring", 0x00C5));
	ret.insert(std::make_pair(L"AElig", 0x00C6));
	ret.insert(std::make_pair(L"Ccedil", 0x00C7));
	ret.insert(std::make_pair(L"Egrave", 0x00C8));
	ret.insert(std::make_pair(L"Eacute", 0x00C9));
	ret.insert(std::make_pair(L"Ecirc", 0x00CA));
	ret.insert(std::make_pair(L"Euml", 0x00CB));
	ret.insert(std::make_pair(L"Igrave", 0x00CC));
	ret.insert(std::make_pair(L"Iacute", 0x00CD));
	ret.insert(std::make_pair(L"Icirc", 0x00CE));
	ret.insert(std::make_pair(L"Iuml", 0x00CF));
	ret.insert(std::make_pair(L"ETH", 0x00D0));
	ret.insert(std::make_pair(L"Ntilde", 0x00D1));
	ret.insert(std::make_pair(L"Ograve", 0x00D2));
	ret.insert(std::make_pair(L"Oacute", 0x00D3));
	ret.insert(std::make_pair(L"Ocirc", 0x00D4));
	ret.insert(std::make_pair(L"Otilde", 0x00D5));
	ret.insert(std::make_pair(L"Ouml", 0x00D6));
	ret.insert(std::make_pair(L"times", 0x00D7));
	ret.insert(std::make_pair(L"Oslash", 0x00D8));
	ret.insert(std::make_pair(L"Ugrave", 0x00D9));
	ret.insert(std::make_pair(L"Uacute", 0x00DA));
	ret.insert(std::make_pair(L"Ucirc", 0x00DB));
	ret.insert(std::make_pair(L"Uuml", 0x00DC));
	ret.insert(std::make_pair(L"Yacute", 0x00DD));
	ret.insert(std::make_pair(L"THORN", 0x00DE));
	ret.insert(std::make_pair(L"szlig", 0x00DF));
	ret.insert(std::make_pair(L"agrave", 0x00E0));
	ret.insert(std::make_pair(L"aacute", 0x00E1));
	ret.insert(std::make_pair(L"acirc", 0x00E2));
	ret.insert(std::make_pair(L"atilde", 0x00E3));
	ret.insert(std::make_pair(L"auml", 0x00E4));
	ret.insert(std::make_pair(L"aring", 0x00E5));
	ret.insert(std::make_pair(L"aelig", 0x00E6));
	ret.insert(std::make_pair(L"ccedil", 0x00E7));
	ret.insert(std::make_pair(L"egrave", 0x00E8));
	ret.insert(std::make_pair(L"eacute", 0x00E9));
	ret.insert(std::make_pair(L"ecirc", 0x00EA));
	ret.insert(std::make_pair(L"euml", 0x00EB));
	ret.insert(std::make_pair(L"igrave", 0x00EC));
	ret.insert(std::make_pair(L"iacute", 0x00ED));
	ret.insert(std::make_pair(L"icirc", 0x00EE));
	ret.insert(std::make_pair(L"iuml", 0x00EF));
	ret.insert(std::make_pair(L"eth", 0x00F0));
	ret.insert(std::make_pair(L"ntilde", 0x00F1));
	ret.insert(std::make_pair(L"ograve", 0x00F2));
	ret.insert(std::make_pair(L"oacute", 0x00F3));
	ret.insert(std::make_pair(L"ocirc", 0x00F4));
	ret.insert(std::make_pair(L"otilde", 0x00F5));
	ret.insert(std::make_pair(L"ouml", 0x00F6));
	ret.insert(std::make_pair(L"divide", 0x00F7));
	ret.insert(std::make_pair(L"oslash", 0x00F8));
	ret.insert(std::make_pair(L"ugrave", 0x00F9));
	ret.insert(std::make_pair(L"uacute", 0x00FA));
	ret.insert(std::make_pair(L"ucirc", 0x00FB));
	ret.insert(std::make_pair(L"uuml", 0x00FC));
	ret.insert(std::make_pair(L"yacute", 0x00FD));
	ret.insert(std::make_pair(L"thorn", 0x00FE));
	ret.insert(std::make_pair(L"yuml", 0x00FF));
	ret.insert(std::make_pair(L"OElig", 0x0152));
	ret.insert(std::make_pair(L"oelig", 0x0153));
	ret.insert(std::make_pair(L"Scaron", 0x0160));
	ret.insert(std::make_pair(L"scaron", 0x0161));
	ret.insert(std::make_pair(L"Yuml", 0x0178));
	ret.insert(std::make_pair(L"fnof", 0x0192));
	ret.insert(std::make_pair(L"circ", 0x02C6));
	ret.insert(std::make_pair(L"tilde", 0x02DC));
	ret.insert(std::make_pair(L"Alpha", 0x0391));
	ret.insert(std::make_pair(L"Beta", 0x0392));
	ret.insert(std::make_pair(L"Gamma", 0x0393));
	ret.insert(std::make_pair(L"Delta", 0x0394));
	ret.insert(std::make_pair(L"Epsilon", 0x0395));
	ret.insert(std::make_pair(L"Zeta", 0x0396));
	ret.insert(std::make_pair(L"Eta", 0x0397));
	ret.insert(std::make_pair(L"Theta", 0x0398));
	ret.insert(std::make_pair(L"Iota", 0x0399));
	ret.insert(std::make_pair(L"Kappa", 0x039A));
	ret.insert(std::make_pair(L"Lambda", 0x039B));
	ret.insert(std::make_pair(L"Mu", 0x039C));
	ret.insert(std::make_pair(L"Nu", 0x039D));
	ret.insert(std::make_pair(L"Xi", 0x039E));
	ret.insert(std::make_pair(L"Omicron", 0x039F));
	ret.insert(std::make_pair(L"Pi", 0x03A0));
	ret.insert(std::make_pair(L"Rho", 0x03A1));
	ret.insert(std::make_pair(L"Sigma", 0x03A3));
	ret.insert(std::make_pair(L"Tau", 0x03A4));
	ret.insert(std::make_pair(L"Upsilon", 0x03A5));
	ret.insert(std::make_pair(L"Phi", 0x03A6));
	ret.insert(std::make_pair(L"Chi", 0x03A7));
	ret.insert(std::make_pair(L"Psi", 0x03A8));
	ret.insert(std::make_pair(L"Omega", 0x03A9));
	ret.insert(std::make_pair(L"alpha", 0x03B1));
	ret.insert(std::make_pair(L"beta", 0x03B2));
	ret.insert(std::make_pair(L"gamma", 0x03B3));
	ret.insert(std::make_pair(L"delta", 0x03B4));
	ret.insert(std::make_pair(L"epsilon", 0x03B5));
	ret.insert(std::make_pair(L"zeta", 0x03B6));
	ret.insert(std::make_pair(L"eta", 0x03B7));
	ret.insert(std::make_pair(L"theta", 0x03B8));
	ret.insert(std::make_pair(L"iota", 0x03B9));
	ret.insert(std::make_pair(L"kappa", 0x03BA));
	ret.insert(std::make_pair(L"lambda", 0x03BB));
	ret.insert(std::make_pair(L"mu", 0x03BC));
	ret.insert(std::make_pair(L"nu", 0x03BD));
	ret.insert(std::make_pair(L"xi", 0x03BE));
	ret.insert(std::make_pair(L"omicron", 0x03BF));
	ret.insert(std::make_pair(L"pi", 0x03C0));
	ret.insert(std::make_pair(L"rho", 0x03C1));
	ret.insert(std::make_pair(L"sigmaf", 0x03C2));
	ret.insert(std::make_pair(L"sigma", 0x03C3));
	ret.insert(std::make_pair(L"tau", 0x03C4));
	ret.insert(std::make_pair(L"upsilon", 0x03C5));
	ret.insert(std::make_pair(L"phi", 0x03C6));
	ret.insert(std::make_pair(L"chi", 0x03C7));
	ret.insert(std::make_pair(L"psi", 0x03C8));
	ret.insert(std::make_pair(L"omega", 0x03C9));
	ret.insert(std::make_pair(L"thetasym", 0x03D1));
	ret.insert(std::make_pair(L"upsih", 0x03D2));
	ret.insert(std::make_pair(L"piv", 0x03D6));
	ret.insert(std::make_pair(L"ensp", 0x2002));
	ret.insert(std::make_pair(L"emsp", 0x2003));
	ret.insert(std::make_pair(L"thinsp", 0x2009));
	ret.insert(std::make_pair(L"zwnj", 0x200C));
	ret.insert(std::make_pair(L"zwj", 0x200D));
	ret.insert(std::make_pair(L"lrm", 0x200E));
	ret.insert(std::make_pair(L"rlm", 0x200F));
	ret.insert(std::make_pair(L"ndash", 0x2013));
	ret.insert(std::make_pair(L"mdash", 0x2014));
	ret.insert(std::make_pair(L"lsquo", 0x2018));
	ret.insert(std::make_pair(L"rsquo", 0x2019));
	ret.insert(std::make_pair(L"sbquo", 0x201A));
	ret.insert(std::make_pair(L"ldquo", 0x201C));
	ret.insert(std::make_pair(L"rdquo", 0x201D));
	ret.insert(std::make_pair(L"bdquo", 0x201E));
	ret.insert(std::make_pair(L"dagger", 0x2020));
	ret.insert(std::make_pair(L"Dagger", 0x2021));
	ret.insert(std::make_pair(L"bull", 0x2022));
	ret.insert(std::make_pair(L"hellip", 0x2026));
	ret.insert(std::make_pair(L"permil", 0x2030));
	ret.insert(std::make_pair(L"prime", 0x2032));
	ret.insert(std::make_pair(L"Prime", 0x2033));
	ret.insert(std::make_pair(L"lsaquo", 0x2039));
	ret.insert(std::make_pair(L"rsaquo", 0x203A));
	ret.insert(std::make_pair(L"oline", 0x203E));
	ret.insert(std::make_pair(L"frasl", 0x2044));
	ret.insert(std::make_pair(L"euro", 0x20AC));
	ret.insert(std::make_pair(L"image", 0x2111));
	ret.insert(std::make_pair(L"weierp", 0x2118));
	ret.insert(std::make_pair(L"real", 0x211C));
	ret.insert(std::make_pair(L"trade", 0x2122));
	ret.insert(std::make_pair(L"alefsym", 0x2135));
	ret.insert(std::make_pair(L"larr", 0x2190));
	ret.insert(std::make_pair(L"uarr", 0x2191));
	ret.insert(std::make_pair(L"rarr", 0x2192));
	ret.insert(std::make_pair(L"darr", 0x2193));
	ret.insert(std::make_pair(L"harr", 0x2194));
	ret.insert(std::make_pair(L"crarr", 0x21B5));
	ret.insert(std::make_pair(L"lArr", 0x21D0));
	ret.insert(std::make_pair(L"uArr", 0x21D1));
	ret.insert(std::make_pair(L"rArr", 0x21D2));
	ret.insert(std::make_pair(L"dArr", 0x21D3));
	ret.insert(std::make_pair(L"hArr", 0x21D4));
	ret.insert(std::make_pair(L"forall", 0x2200));
	ret.insert(std::make_pair(L"part", 0x2202));
	ret.insert(std::make_pair(L"exist", 0x2203));
	ret.insert(std::make_pair(L"empty", 0x2205));
	ret.insert(std::make_pair(L"nabla", 0x2207));
	ret.insert(std::make_pair(L"isin", 0x2208));
	ret.insert(std::make_pair(L"notin", 0x2209));
	ret.insert(std::make_pair(L"ni", 0x220B));
	ret.insert(std::make_pair(L"prod", 0x220F));
	ret.insert(std::make_pair(L"sum", 0x2211));
	ret.insert(std::make_pair(L"minus", 0x2212));
	ret.insert(std::make_pair(L"lowast", 0x2217));
	ret.insert(std::make_pair(L"radic", 0x221A));
	ret.insert(std::make_pair(L"prop", 0x221D));
	ret.insert(std::make_pair(L"infin", 0x221E));
	ret.insert(std::make_pair(L"ang", 0x2220));
	ret.insert(std::make_pair(L"and", 0x2227));
	ret.insert(std::make_pair(L"or", 0x2228));
	ret.insert(std::make_pair(L"cap", 0x2229));
	ret.insert(std::make_pair(L"cup", 0x222A));
	ret.insert(std::make_pair(L"int", 0x222B));
	ret.insert(std::make_pair(L"there4", 0x2234));
	ret.insert(std::make_pair(L"sim", 0x223C));
	ret.insert(std::make_pair(L"cong", 0x2245));
	ret.insert(std::make_pair(L"asymp", 0x2248));
	ret.insert(std::make_pair(L"ne", 0x2260));
	ret.insert(std::make_pair(L"equiv", 0x2261));
	ret.insert(std::make_pair(L"le", 0x2264));
	ret.insert(std::make_pair(L"ge", 0x2265));
	ret.insert(std::make_pair(L"sub", 0x2282));
	ret.insert(std::make_pair(L"sup", 0x2283));
	ret.insert(std::make_pair(L"nsub", 0x2284));
	ret.insert(std::make_pair(L"sube", 0x2286));
	ret.insert(std::make_pair(L"supe", 0x2287));
	ret.insert(std::make_pair(L"oplus", 0x2295));
	ret.insert(std::make_pair(L"otimes", 0x2297));
	ret.insert(std::make_pair(L"perp", 0x22A5));
	ret.insert(std::make_pair(L"sdot", 0x22C5));
	ret.insert(std::make_pair(L"vellip", 0x22EE));
	ret.insert(std::make_pair(L"lceil", 0x2308));
	ret.insert(std::make_pair(L"rceil", 0x2309));
	ret.insert(std::make_pair(L"lfloor", 0x230A));
	ret.insert(std::make_pair(L"rfloor", 0x230B));
	ret.insert(std::make_pair(L"lang", 0x2329));
	ret.insert(std::make_pair(L"rang", 0x232A));
	ret.insert(std::make_pair(L"loz", 0x25CA));
	ret.insert(std::make_pair(L"spades", 0x2660));
	ret.insert(std::make_pair(L"clubs", 0x2663));
	ret.insert(std::make_pair(L"hearts", 0x2665));
	ret.insert(std::make_pair(L"diams", 0x2666));

	return ret;
}

bool utility::is_http_https_url(const std::wstring& url)
{
	return url.length() > 10
		&& (boost::istarts_with(url, L"http://") || boost::istarts_with(url, L"https://"));
}

bool utility::get_proxy_info(std::wstring& proxy_address, std::wstring& proxy_bypass)
{
	proxy_address.clear();
	proxy_bypass.clear();

	WINHTTP_CURRENT_USER_IE_PROXY_CONFIG proxy_info;
	if(FALSE == WinHttpGetIEProxyConfigForCurrentUser(&proxy_info))
		return false;

	if(proxy_info.lpszProxy)
		proxy_address = proxy_info.lpszProxy;

	if(proxy_info.lpszProxyBypass)
		proxy_bypass = proxy_info.lpszProxyBypass;

	return !proxy_address.empty();
}

uint32_t utility::get_frame_count(Gdiplus::Image& image)
{
	UINT frame_dimensions_count = image.GetFrameDimensionsCount();
	boost::scoped_array<GUID> dimension_ids(new GUID[frame_dimensions_count]);

	image.GetFrameDimensionsList(dimension_ids.get(), frame_dimensions_count);
	UINT frame_count = image.GetFrameCount(&dimension_ids[0]);

	return frame_count;
}

const std::vector<uint32_t> utility::get_frame_delays(Gdiplus::Image& image, uint32_t frame_count)
{
	UINT bufsize = image.GetPropertyItemSize(PropertyTagFrameDelay);
	Gdiplus::PropertyItem* items = reinterpret_cast<Gdiplus::PropertyItem*>(malloc(bufsize));
	if(!items)
		throw std::bad_alloc();

	std::vector<uint32_t> ret;

	try
	{
		if(Gdiplus::Ok != image.GetPropertyItem(PropertyTagFrameDelay, bufsize, items))
			throw sp_exception(sp_exception::internal_error, L"Unable to get frame delays");

		for(uint32_t i = 0; i != frame_count; ++i)
			ret.push_back((reinterpret_cast<const UINT*>(items->value))[i] * 10);
	}
	catch(const std::exception&)
	{
		free(items);
		throw;
	}

	free(items);

	return ret;
}

std::auto_ptr<unpacked_image> utility::scale_image(Gdiplus::Image& source_image, int max_size, int max_gif_size)
{
	int src_width = source_image.GetWidth();
	int src_height = source_image.GetHeight();

	int dst_width  = src_width;
	int dst_height = src_height;

	uint32_t frame_count = get_frame_count(source_image);
	
	std::vector<uint32_t> frame_delays;
	if(frame_count > 1)
	{
		frame_delays = get_frame_delays(source_image, frame_count);
		max_size = max_gif_size;
	}
	else
	{
		frame_delays.push_back(0);
	}

	if(dst_width > max_size || dst_height > max_size)
	{
		if(dst_width > dst_height)
		{
			dst_height = static_cast<int>(dst_height * (static_cast<float>(max_size) / dst_width));
			dst_width = max_size;
		}
		else
		{
			dst_width = static_cast<int>(dst_width * (static_cast<float>(max_size) / dst_height));
			dst_height = max_size;
		}
	}
	
	std::auto_ptr<unpacked_image> ret(new unpacked_image);
	GUID frame_dimension_time_guid = Gdiplus::FrameDimensionTime;
	for(uint32_t i = 0; i != frame_count; ++i)
	{
		std::auto_ptr<Gdiplus::Bitmap> result(new Gdiplus::Bitmap(dst_width, dst_height, PixelFormat32bppARGB));

		result->SetResolution(source_image.GetHorizontalResolution(),
			source_image.GetVerticalResolution());

		Gdiplus::Graphics graphics(result.get());

		graphics.Clear(Gdiplus::Color::Transparent);

		graphics.SetInterpolationMode(Gdiplus::InterpolationModeBicubic);
		//graphics.SetInterpolationMode(Gdiplus::InterpolationModeNearestNeighbor);

		Gdiplus::Rect dest(0, 0, dst_width, dst_height);

		source_image.SelectActiveFrame(&frame_dimension_time_guid, i);

		graphics.DrawImage(&source_image, dest, 0, 0, src_width, src_height, Gdiplus::UnitPixel);
		
		ret->add_image(result.release(), frame_delays[i]);
	}
	
	return ret;
}

std::auto_ptr<Gdiplus::Image> utility::load_from_resource(const wchar_t* resource_name, const wchar_t* resource_type, bool use_embedded_color_management)
{
	HINSTANCE inst = configuration::get_instance();
	HRSRC res = FindResourceW(inst, resource_name, resource_type);
	if(!res)
		throw sp_exception(sp_exception::internal_error, L"Unable to find resource");

	DWORD res_size = SizeofResource(inst, res);
	HGLOBAL res_data = LoadResource(inst, res);
	if(!res_data)
		throw sp_exception(sp_exception::internal_error, L"Unable to find resource");

	LPVOID res_buf = LockResource(res_data);

	byte_stream* stream =
		byte_stream::create(std::string(reinterpret_cast<const char*>(res_buf), res_size));

	return std::auto_ptr<Gdiplus::Image>(
		new istream_owner_image(stream, use_embedded_color_management));
}

const std::wstring utility::get_module_name_from_address(const void* address)
{
	MEMORY_BASIC_INFORMATION mbi = {};
	if(!VirtualQuery(address, &mbi, sizeof(mbi)))
		throw sp_exception(sp_exception::internal_error, L"Unable to get address information");

	HMODULE mod = reinterpret_cast<HMODULE>(mbi.AllocationBase);

	wchar_t module_name[MAX_PATH] = {};
	if(!GetModuleBaseNameW(GetCurrentProcess(), mod, module_name, MAX_PATH))
		throw sp_exception(sp_exception::internal_error, L"Unable to get module name");

	return module_name;
}

const utility::mouse_info utility::get_mouse_info()
{
	CURSORINFO ci = {};
	ci.cbSize = sizeof(ci);
	if(!GetCursorInfo(&ci))
		throw sp_exception(sp_exception::internal_error, L"Unable to get cursor info");

	mouse_info ret;

	ret.is_shown = (ci.flags & CURSOR_SHOWING) != 0;
	ret.x = ci.ptScreenPos.x;
	ret.y = ci.ptScreenPos.y;

	ret.is_hand = ci.hCursor == hand_cursor_;

	ret.left_pressed = (GetKeyState(VK_LBUTTON) & 0xff00) != 0;
	ret.right_pressed = (GetKeyState(VK_RBUTTON) & 0xff00) != 0;
	ret.middle_pressed = (GetKeyState(VK_MBUTTON) & 0xff00) != 0;

	return ret;
}

const std::string utility::get_html_page_title(const std::string& content)
{
	boost::match_results<std::string::const_iterator> match;
	if(boost::regex_search(content, match, title_regex_))
		return match[1].str();

	return std::string();
}

const std::wstring utility::trim(const std::wstring& content)
{
	std::wstring ret(content);
	boost::replace_all_regex(ret, space_regex_, std::wstring(L" "));
	boost::trim(ret);
	return ret;
}

std::auto_ptr<Gdiplus::Bitmap> utility::draw_text(const std::wstring& text, uint32_t max_width)
{
	Gdiplus::RectF bounding_box;
	Gdiplus::Font font(L"Arial", 12);

	{
		Gdiplus::Bitmap buf(1, 1);
		Gdiplus::Graphics graphics(&buf);
		const Gdiplus::RectF max_size(0, 0,
			static_cast<Gdiplus::REAL>(max_width), 0);
		graphics.MeasureString(text.c_str(), -1, &font, max_size, &bounding_box);
	}

	const uint32_t indent = 10;

	INT text_width = static_cast<INT>(ceil(bounding_box.Width));
	INT text_height = static_cast<INT>(ceil(bounding_box.Height));
	std::auto_ptr<Gdiplus::Bitmap> image(
		new Gdiplus::Bitmap(text_width + indent, text_height + indent));
	Gdiplus::Graphics graphics(image.get());
	Gdiplus::SolidBrush brush(Gdiplus::Color(0, 0, 0));

	Gdiplus::Pen pen(&brush);
	Gdiplus::Rect bounding_rect(0, 0, text_width + indent - 1, text_height + indent - 1);
	graphics.DrawRectangle(&pen, bounding_rect);

	const Gdiplus::RectF layout(indent / 2, indent / 2, bounding_box.Width, 0);
	const Gdiplus::StringFormat format;
	graphics.SetTextRenderingHint(Gdiplus::TextRenderingHintAntiAlias);
	graphics.DrawString(text.c_str(), -1, &font, layout, &format, &brush);

	return image;
}

const std::string utility::get_html_page_charset(const std::string& headers, const std::string& content)
{
	boost::match_results<std::string::const_iterator> header_match;
	std::string ret;
	std::string::const_iterator begin = headers.begin();
	std::string::const_iterator end = headers.end();
	while(boost::regex_search(begin, end, header_match, header_charset_regex_))
	{
		ret = header_match[1].str();
		begin = header_match[0].second;
	}

	if(ret.empty())
	{
		boost::match_results<std::string::const_iterator> content_match;
		if(boost::regex_search(content, content_match, meta_charset_regex_))
			ret = content_match[1].str();
	}

	return ret;
}

const std::wstring utility::convert_to_unicode(const std::string& text, const std::string& charset)
{
	std::wstring ret;
	if(text.empty())
		return ret;

	iconv_t conv = iconv_open("UCS-2LE", charset.c_str());
	if(conv == reinterpret_cast<iconv_t>(-1))
		throw sp_exception(sp_exception::internal_error, L"Unable to open iconv with specified charset");

	ret.resize(text.length());

	size_t inbytesleft = text.length();
	size_t outbytesleft = text.length() * sizeof(std::wstring::value_type);
	const char* in_pos = text.c_str();
	wchar_t* out_pos = &ret[0];

	size_t result = iconv(conv,
		&in_pos,
		&inbytesleft,
		reinterpret_cast<char**>(&out_pos),
		&outbytesleft);
	iconv_close(conv);

	if(result == static_cast<size_t>(-1))
		throw sp_exception(sp_exception::internal_error, L"Iconv conversion error");

	if(outbytesleft)
		ret.resize(ret.length() - outbytesleft / sizeof(std::wstring::value_type));

	return ret;
}

HWND utility::get_main_window(uint32_t process_id)
{
	handle_data data;
	data.process_id = process_id;
	data.best_handle = 0;
	EnumWindows(enum_windows_callback, reinterpret_cast<LPARAM>(&data));
	return data.best_handle;
}

BOOL utility::is_main_window(HWND handle)
{   
	return GetWindow(handle, GW_OWNER) == 0 && IsWindowVisible(handle);
}

BOOL CALLBACK utility::enum_windows_callback(HWND handle, LPARAM lParam)
{
	handle_data& data = *reinterpret_cast<handle_data*>(lParam);
	DWORD process_id = 0;
	GetWindowThreadProcessId(handle, &process_id);
	if(data.process_id != process_id || !is_main_window(handle))
		return TRUE;

	data.best_handle = handle;
	return FALSE;   
}

void utility::init_common_controls()
{
	INITCOMMONCONTROLSEX init;
	init.dwSize = sizeof(init);
	init.dwICC = ICC_LINK_CLASS | ICC_UPDOWN_CLASS | ICC_INTERNET_CLASSES;
	InitCommonControlsEx(&init);
}

int utility::add_encoding(unsigned int namescount, const char* const * names, void* data)
{
	encoding_list* list = reinterpret_cast<encoding_list*>(data);
	for (unsigned int i = 0; i != namescount; ++i)
		list->push_back(names[i]);

	return 0;
}

const utility::encoding_list utility::get_supported_encoding_list()
{
	encoding_list ret;
	iconvlist(add_encoding, reinterpret_cast<void*>(&ret));
	return ret;
}

const std::wstring utility::replace_html_entities(const std::wstring& content)
{
	enum code_mode
	{
		mode_string,
		mode_decimal,
		mode_hex
	};

	std::wstringstream result;
	
	std::wstringstream code;
	bool code_parse = false;
	code_mode mode = mode_string;

	for(std::wstring::const_iterator it = content.begin(); it != content.end(); ++it)
	{
		wchar_t c = *it;
		if(code_parse)
		{
			if(c == L';')
			{
				std::wstring entity(code.str());
				if(mode == mode_string)
				{
					html_entity_map::const_iterator entity_it = html_entities_.find(entity);
					if(entity_it == html_entities_.end())
						result << L"&" << entity << L";";
					else
						result << (*entity_it).second;
				}
				else if(mode == mode_decimal)
				{
					try
					{
						uint16_t character = boost::lexical_cast<uint16_t>(code.str());
						if(character == 0)
							throw sp_exception(sp_exception::internal_error, L"Incorrect HTML entity sequence");

						result << static_cast<wchar_t>(character);
					}
					catch(const std::exception&)
					{
						result << L"&#" << entity << L";";
					}
				}
				else //hex
				{
					try
					{
						std::wstringstream ss;
						ss.exceptions(std::ios::failbit | std::ios::badbit);
						uint16_t character;
						ss << std::hex << entity.substr(1); //skip first 'x' or 'X'
						ss >> character;
						if(character == 0)
							throw sp_exception(sp_exception::internal_error, L"Incorrect HTML entity sequence");

						result << static_cast<wchar_t>(character);
					}
					catch(const std::exception&)
					{
						result << L"&#" << entity << L";";
					}
				}

				code.str(L"");
				code_parse = false;
			}
			else if(c == L'#' && code.rdbuf()->in_avail() == 0)
			{
				mode = mode_decimal;
			}
			else if((c == L'x' || c == L'X') && mode == mode_decimal && code.rdbuf()->in_avail() == 0)
			{
				code << c;
				mode = mode_hex;
			}
			else if((mode == mode_string && isalnum(c))
				|| (mode == mode_decimal && isdigit(c))
				|| (mode == mode_hex && isxdigit(c)))
			{
				code << c;
			}
			else
			{
				result << L'&';
				if(mode != mode_string)
					result << L'#';

				result << code.str() << c;
				code.str(L"");
				code_parse = false;
			}
		}
		else
		{
			if(c == L'&')
			{
				code_parse = true;
				mode = mode_string;
			}
			else
			{
				result << c;
			}
		}
	}

	if(code_parse)
	{
		result << L"&";
		if(mode != mode_string)
			result << L'#';
		
		result << code.str();
	}

	return result.str();
}

const RECT utility::get_primary_monitor_rect()
{
	RECT ret;
	ret.left = ret.top = 0;
	ret.right = GetSystemMetrics(SM_CXSCREEN);
	ret.bottom = GetSystemMetrics(SM_CYSCREEN);
	return ret;
}

const RECT utility::get_monitor_rect_from_point(int32_t x, int32_t y)
{
	POINT pos;
	pos.x = x;
	pos.y = y;
	HMONITOR monitor = MonitorFromPoint(pos, MONITOR_DEFAULTTONEAREST);
	if(!monitor)
		return get_primary_monitor_rect();

	MONITORINFO monitor_info = {};
	monitor_info.cbSize = sizeof(monitor_info);
	if(!GetMonitorInfo(monitor, &monitor_info))
		return get_primary_monitor_rect();

	return monitor_info.rcWork;
}
