#pragma once
#include <string>
#include <memory>
#include <map>
#include <cstdint>
#include <vector>
#include <Windows.h>
#include <GdiPlus.h>
#include <boost/noncopyable.hpp>
#include <boost/regex.hpp>
#include "unpacked_image.h"

class utility : boost::noncopyable
{
public:
	struct mouse_info
	{
		int32_t x;
		int32_t y;
		bool left_pressed;
		bool middle_pressed;
		bool right_pressed;
		bool is_shown;
		bool is_hand;
	};

	typedef std::vector<std::string> encoding_list;

public:
	//Returns true if using proxy
	static bool get_proxy_info(std::wstring& proxy_address, std::wstring& proxy_bypass);

	static uint32_t get_frame_count(Gdiplus::Image& image);

	static const std::vector<uint32_t> get_frame_delays(Gdiplus::Image& image, uint32_t frame_count);

	static std::auto_ptr<unpacked_image> scale_image(Gdiplus::Image& source_image, int max_size, int max_gif_size);

	static bool is_http_https_url(const std::wstring& url);

	static std::auto_ptr<Gdiplus::Image> load_from_resource(const wchar_t* resource_name,
		const wchar_t* resource_type, bool use_embedded_color_management);

	static const std::wstring get_module_name_from_address(const void* address);

	static const mouse_info get_mouse_info();

	static const std::string get_html_page_title(const std::string& content);

	static const std::string get_html_page_charset(const std::string& headers, const std::string& content);
	
	static std::auto_ptr<Gdiplus::Bitmap> draw_text(const std::wstring& text, uint32_t max_width);

	static const std::wstring convert_to_unicode(const std::string& text, const std::string& charset);

	static HWND get_main_window(uint32_t process_id);

	static void init_common_controls();

	static const encoding_list get_supported_encoding_list();
	
	static const std::wstring replace_html_entities(const std::wstring& content);

	static const std::wstring trim(const std::wstring& content);

	static const RECT get_monitor_rect_from_point(int32_t x, int32_t y);
	static const RECT get_primary_monitor_rect();

private:
	struct handle_data
	{
		unsigned long process_id;
		HWND best_handle;
	};

private:
	utility();
	
	typedef std::map<std::wstring, wchar_t> html_entity_map;
	static const utility::html_entity_map init_html_entities();

	static BOOL is_main_window(HWND handle);
	static BOOL CALLBACK enum_windows_callback(HWND handle, LPARAM lParam);
	static int add_encoding(unsigned int namescount, const char* const * names, void* data);

	static HCURSOR hand_cursor_;
	static boost::regex title_regex_;
	static boost::regex header_charset_regex_;
	static boost::regex meta_charset_regex_;
	static boost::wregex space_regex_;
	static const html_entity_map html_entities_;
};
