#pragma once
#include <string>
#include <cstdint>
#include <utility>
#include <boost/thread/mutex.hpp>
#include <boost/noncopyable.hpp>

//cmake ../ -G "Visual Studio 10" -DHTTP_ONLY=ON -DCURL_STATICLIB=ON -DBUILD_CURL_TESTS=OFF -DOPENSSL_USE_STATIC=1 -DOPENSSL_LIBRARIES="C:\OpenSSL-Win32\lib\VC\static"

class web_content_loader : boost::noncopyable
{
public:
	web_content_loader();

	const std::string load(const std::string& url, std::string* headers);

	void set_timeout(uint32_t timeout);
	void set_connection_timeout(uint32_t connection_timeout);
	void set_max_data_size(uint32_t max_data_size);
	void set_max_redirects(uint32_t max_redirects);
	void set_user_agent(const std::string& user_agent);
	
	void set_proxy(const std::string& proxy_url);
	void set_proxy_auth(const std::string& proxy_login, const std::string& proxy_password);

	void abort();

	bool is_ready() const;

	~web_content_loader();

public:
	enum proxy_type
	{
		proxy_http,
		proxy_socks4,
		proxy_socks4a,
		proxy_socks5,
		proxy_socks5h
	};

	typedef std::pair<std::wstring, proxy_type> proxy_info;

	static const proxy_info get_proxy_info(const std::wstring& proxy_string);

private:
	void cleanup();

	static size_t data_callback(void *ptr, size_t size, size_t nmemb, void *stream);
	static int progress_callback(void *clientp, double dltotal, double dlnow, double ultotal, double ulnow);

	void* curl_;

	bool aborted_;
	bool ready_;
	boost::mutex abort_mutex_;
	mutable boost::mutex ready_mutex_;

	uint32_t max_data_size_;

	static uint32_t global_init_;
	static boost::mutex global_init_mutex_;
};
